/*----------------------------------------------------------------------------*
 * 
 *	file: ppmout.c
 * 
 *	function: vector or array to ppm file routines 
 *	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
 *	creation: Thu Sep 25 21:50:57 CDT 1997
 * 
 *	$Revision: 1.3 $ $Date: 1999/02/09 07:42:25 $
 *	$Log: ppmout.cc,v $
 *	Revision 1.3  1999/02/09 07:42:25  yschoe
 *	*** empty log message ***
 *
 *	Revision 1.2  1998/09/04 03:36:37  yschoe
 *	fixed all warnings generated by -Wall
 *
 *	Revision 1.1  1998/07/23 01:07:05  yschoe
 *	jul22 stable version (except netlayer)
 *
 *	Revision 1.1  1998/07/07 08:36:47  yschoe
 *	Initial revision
 *
 *----------------------------------------------------------------------------*/
#include "ppmout.h"

static struct color_t   {
        int     red;
        int     green;
        int     blue;
} colors[256]; 


/*----------------------------------------------------------------------------*
 *      errorexit()
 *        - display function where it was called and a message and exit
 *----------------------------------------------------------------------------*/
static void    errorexit(char *func, char *msg)
{
  fprintf(stderr,"%s(): %s\n",func,msg);
  exit(1);
}

/*----------------------------------------------------------------------------*
 *	init_color_table() 
 *	  - initialize color RGB table
 *----------------------------------------------------------------------------*/
void	init_color_table()
{
 int rangeby3, multiplier, i ; 

 rangeby3   = (int) (COLOR_RANGE/3);
 multiplier = (int) (256/rangeby3);

 /* printf("rangeby3 = %d\n",rangeby3); */

 for(i= 0; i<COLOR_RANGE; i++ )   
 {
   	colors[i].green = 0;
   	colors[i].blue  = 0;
   	if ( i < rangeby3 )	/* black --> red */
	{
     		colors[i].red = i*multiplier;
	}
   	else{
     		colors[i].red = 255;
	
     		if ( i < 2*rangeby3) /* red --> yellow */
		{
       			colors[i].green = (i-rangeby3)*multiplier;
		}
     		else{  /* yellow --> white */
       			colors[i].green = 255;
       			colors[i].blue  = (i - 2*rangeby3)*multiplier;
			/* printf("[%d]\n",colors[i].blue); */
     		}
   	}
  }
}

void   vector_to_ppm (const char *filename, REAL *vector, int row, int col) {

  vector_to_ppm_select(filename,vector,row,col,"rgb");

}

/*----------------------------------------------------------------------------*
 *	vector_to_ppm_select() 
 *	  - save vector of REAL values to a color coded ppm file
 *	  - if select == "hsv", save in hsv format. 
 *	    if select == "rgb", save in rgb format.
 *----------------------------------------------------------------------------*/

void	vector_to_ppm_select (const char *filename, REAL *vector, 
	int row, int col, char *select)
{
  FILE 	*fp;
  int	i, j, index;
  REAL  min=999999.9, max=-999999.9, diff;

  int	init_flag=0;

  /* init colormap if not initialized */
  if (!init_flag){
  	init_color_table();
	init_flag = 1;
  }

  /* open output file */
  if (!(fp=fopen(filename,"w"))) {
	errorexit("vector_to_ppm", "cannot create/write file");
  }

  /* init header */
  fprintf(fp,"P3\n#%s\n",filename);
  fprintf(fp,"%d %d\n",col, row);
  fprintf(fp,"256\n");

  /* find Min and Max */
  for (i=0; i<row; ++i)
  {
	for (j=0; j<col; ++j)
	{
		if (vector[i*col+j]> max) max = vector[i*col+j];
		if (vector[i*col+j]< min) min = vector[i*col+j];
	}
  }
  diff = max - min;

  /* print out color values */
  for (i=0; i<row; ++i)
  {
	for (j=0; j<col; ++j)
	{
		index = (int)((vector[i*col+j]-min)*255/diff);
		fprintf(fp,"%d %d %d   ", 
				colors[index].red,
				colors[index].green,
				colors[index].blue
		);
		/* if (j%20==0) fprintf(fp,"\n"); */
	}
	fprintf(fp,"\n");
   }
   fclose(fp);
}

#ifdef TESTDRIVE
/*----------------------------------------------------------------------------*
 *
 *	main() 
 *	  -  main driver function
 *
 *----------------------------------------------------------------------------*/
main()
{
  REAL  mat[256];
  int 	i=0;

  for (i=0; i<256; ++i)
  {
	mat[i]=(REAL)(i/256.0);
	/* printf ("%lf ",mat[i]); */
  }

  vector_to_ppm("scale.ppm",(REAL *)mat,256,1);
}
#endif
