## Copyright (C) 2000 uschoe
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## usage: N = samplemat(M,rfsize,skip)
##
## Returns p x q matrix where p is the number of samples drawn from M
## and q is the vector length of each sample. skip is the interval
## for the successive RF locations.
## 
##
## M is a 2D matrix, and a small square area of rfsize x rfsize is
## taken and the sample is appended in N as a new row.
##

## Author: Yoonsuck Choe <yschoe@cs.utexas.edu>

function N = samplemat(M,rfsize,skip)

	c = columns(M);
	r = rows(M);

	# calc starting and ending locations for the RF scan
	sc = 1; ec = c-rfsize+1;
	sr = 1; er = r-rfsize+1;

	N = [ vec(M(1:(1+rfsize-1),1:(1+rfsize-1)))' ];

	for i=sr:skip:er
		for j=sc:skip:ec
			N = [ N; vec(M(i:(i+rfsize-1),j:(j+rfsize-1)))' ];
		endfor
	endfor

endfunction
